package com.ruoyi.system.utils.dbsync.meta;

import java.util.Iterator;
import java.sql.ResultSet;
import com.ruoyi.system.utils.dbsync.util.SqlUtil;
import java.sql.SQLException;
import java.sql.DriverManager;
import com.alibaba.druid.util.JdbcUtils;
import java.util.LinkedHashMap;
import java.util.Map;
import java.sql.Connection;

public class MetaData
{
    private Connection conn;
    private String user;
    private String password;
    private String schema;
    private String jdbcUrl;
    private Map<String, Table> tables;
    
    public MetaData() {
        this.tables = new LinkedHashMap<String, Table>();
    }
    
    public Boolean init() {
        try {
            Class.forName(JdbcUtils.getDriverClassName(this.jdbcUrl));
            this.conn = DriverManager.getConnection(this.jdbcUrl, this.user, this.password);
            this.getTablesFromDb();
            this.getCreateTables();
            this.getAllTableColumns();
            this.getAllTableKeys();
            return Boolean.TRUE;
        }
        catch (final Exception e) {
            return Boolean.FALSE;
        }
    }
    
    public Boolean connect() {
        try {
            Class.forName(JdbcUtils.getDriverClassName(this.jdbcUrl));
            this.conn = DriverManager.getConnection(this.jdbcUrl, this.user, this.password);
            return Boolean.TRUE;
        }
        catch (final Exception e) {
            return Boolean.FALSE;
        }
    }
    
    public void setAutoCommit(final Boolean auto) throws SQLException {
        this.conn.setAutoCommit(auto);
    }
    
    public void Commit() throws SQLException {
        this.conn.commit();
    }
    
    private void getTablesFromDb() throws SQLException {
        final ResultSet rs = SqlUtil.executeSql(this.conn, "show tables from `" + this.schema + "`");
        while (rs.next()) {
            final Table table = new Table();
            table.setTableName(rs.getString("Tables_in_" + this.schema));
            this.tables.put(table.getTableName(), table);
        }
    }
    
    private void getCreateTables() throws SQLException {
        for (final Table table : this.tables.values()) {
            this.getCreateTable(table);
        }
    }
    
    private void getCreateTable(final Table table) throws SQLException {
        final ResultSet rs = SqlUtil.executeSql(this.conn, "show create table `" + this.schema + "`." + table.getTableName());
        while (rs.next()) {
            table.setCreateTable(rs.getString("Create Table"));
        }
    }
    
    private void getAllTableColumns() throws SQLException {
        for (final Table table : this.tables.values()) {
            this.getTableColumns(table);
        }
    }
    
    private void getTableColumns(final Table table) throws SQLException {
        final ResultSet rs = SqlUtil.executeSql(this.conn, "select COLUMN_NAME,COLUMN_TYPE,IS_NULLABLE,COLUMN_DEFAULT,COLUMN_COMMENT,EXTRA from information_schema.columns where TABLE_SCHEMA='" + this.schema + "' and TABLE_NAME='" + table.getTableName() + "' order by ORDINAL_POSITION asc");
        while (rs.next()) {
            final Column column = new Column();
            column.setName(rs.getString("COLUMN_NAME"));
            column.setType(rs.getString("COLUMN_TYPE"));
            column.setDefaultValue(rs.getString("COLUMN_DEFAULT"));
            column.setIsNull(rs.getString("IS_NULLABLE"));
            column.setExtra(rs.getString("EXTRA"));
            column.setComment(rs.getString("COLUMN_COMMENT"));
            table.getColumns().put(column.getName(), column);
        }
    }
    
    private void getAllTableKeys() throws SQLException {
        for (final Table table : this.tables.values()) {
            this.getTableKeys(table);
        }
    }
    
    private void getTableKeys(final Table table) throws SQLException {
        final ResultSet rs = SqlUtil.executeSql(this.conn, "show keys from `" + this.schema + "`." + table.getTableName());
        Index last = null;
        final Index tmp = null;
        while (rs.next()) {
            final String keyName = rs.getString("Key_name");
            if (last == null || !keyName.equals(last.getIndexName())) {
                last = new Index();
                last.setIndexName(keyName);
                last.getColumns().add(rs.getString("Column_name"));
                last.setNotUnique(rs.getString("Non_unique"));
                table.getIndexes().put(last.getIndexName(), last);
            }
            else {
                last.getColumns().add(rs.getString("Column_name"));
            }
        }
    }
    
    public String getJdbcUrl() {
        return this.jdbcUrl;
    }
    
    public void setJdbcUrl(final String jdbcUrl) {
        this.jdbcUrl = jdbcUrl;
    }
    
    public String getSchema() {
        return this.schema;
    }
    
    public void setSchema(final String schema) {
        this.schema = schema;
    }
    
    public String getUser() {
        return this.user;
    }
    
    public void setUser(final String user) {
        this.user = user;
    }
    
    public String getPassword() {
        return this.password;
    }
    
    public void setPassword(final String password) {
        this.password = password;
    }
    
    public Connection getConn() {
        return this.conn;
    }
    
    public void setConn(final Connection conn) {
        this.conn = conn;
    }
    
    public Map<String, Table> getTables() {
        return this.tables;
    }
    
    public void setTables(final Map<String, Table> tables) {
        this.tables = tables;
    }
}
